@@ -12,13 +12,13 @@ from __future__ import annotations
 import typing as t
 from collections import defaultdict
 from contextlib import contextmanager
+from inspect import iscoroutinefunction
 from warnings import warn
 from weakref import WeakValueDictionary
 
 from blinker._utilities import annotatable_weakref
 from blinker._utilities import hashable_identity
 from blinker._utilities import IdentityType
-from blinker._utilities import is_coroutine_function
 from blinker._utilities import lazy_property
 from blinker._utilities import reference
 from blinker._utilities import symbol
@@ -32,13 +32,17 @@ if t.TYPE_CHECKING:
     T = t.TypeVar("T")
     P = te.ParamSpec("P")
 
-    AsyncWrapperType = t.Callable[[t.Callable[P, T]], t.Callable[P, t.Awaitable[T]]]
-    SyncWrapperType = t.Callable[[t.Callable[P, t.Awaitable[T]]], t.Callable[P, T]]
+    AsyncWrapperType = t.Callable[[t.Callable[P, t.Awaitable[T]]], t.Callable[P, T]]
+    SyncWrapperType = t.Callable[[t.Callable[P, T]], t.Callable[P, t.Awaitable[T]]]
 
 ANY = symbol("ANY")
 ANY.__doc__ = 'Token for "any sender".'
 ANY_ID = 0
 
+# NOTE: We need a reference to cast for use in weakref callbacks otherwise
+#       t.cast may have already been set to None during finalization.
+cast = t.cast
+
 
 class Signal:
     """A notification emitter."""
@@ -47,6 +51,8 @@ class Signal:
     #: without an additional import.
     ANY = ANY
 
+    set_class: type[set] = set
+
     @lazy_property
     def receiver_connected(self) -> Signal:
         """Emitted after each :meth:`connect`.
@@ -99,8 +105,12 @@ class Signal:
         #: any receivers are connected to the signal.
         self.receivers: dict[IdentityType, t.Callable | annotatable_weakref] = {}
         self.is_muted = False
-        self._by_receiver: dict[IdentityType, set[IdentityType]] = defaultdict(set)
-        self._by_sender: dict[IdentityType, set[IdentityType]] = defaultdict(set)
+        self._by_receiver: dict[IdentityType, set[IdentityType]] = defaultdict(
+            self.set_class
+        )
+        self._by_sender: dict[IdentityType, set[IdentityType]] = defaultdict(
+            self.set_class
+        )
         self._weak_senders: dict[IdentityType, annotatable_weakref] = {}
 
     def connect(
@@ -229,10 +239,7 @@ class Signal:
         self.connect(receiver, sender=sender, weak=False)
         try:
             yield None
-        except Exception as e:
-            self.disconnect(receiver)
-            raise e
-        else:
+        finally:
             self.disconnect(receiver)
 
     @contextmanager
@@ -293,11 +300,11 @@ class Signal:
         sender = self._extract_sender(sender)
         results = []
         for receiver in self.receivers_for(sender):
-            if is_coroutine_function(receiver):
+            if iscoroutinefunction(receiver):
                 if _async_wrapper is None:
                     raise RuntimeError("Cannot send to a coroutine function")
                 receiver = _async_wrapper(receiver)
-            result = receiver(sender, **kwargs)  # type: ignore[call-arg]
+            result = receiver(sender, **kwargs)
             results.append((receiver, result))
         return results
 
@@ -325,11 +332,11 @@ class Signal:
         sender = self._extract_sender(sender)
         results = []
         for receiver in self.receivers_for(sender):
-            if not is_coroutine_function(receiver):
+            if not iscoroutinefunction(receiver):
                 if _sync_wrapper is None:
                     raise RuntimeError("Cannot send to a non-coroutine function")
-                receiver = _sync_wrapper(receiver)  # type: ignore[arg-type]
-            result = await receiver(sender, **kwargs)  # type: ignore[call-arg, misc]
+                receiver = _sync_wrapper(receiver)
+            result = await receiver(sender, **kwargs)
             results.append((receiver, result))
         return results
 
@@ -374,7 +381,7 @@ class Signal:
 
     def receivers_for(
         self, sender: t.Any
-    ) -> t.Generator[t.Callable | annotatable_weakref, None, None]:
+    ) -> t.Generator[t.Callable[[t.Any], t.Any], None, None]:
         """Iterate all live receivers listening for *sender*."""
         # TODO: test receivers_for(ANY)
         if self.receivers:
@@ -430,11 +437,11 @@ class Signal:
 
     def _cleanup_receiver(self, receiver_ref: annotatable_weakref) -> None:
         """Disconnect a receiver from all senders."""
-        self._disconnect(t.cast(IdentityType, receiver_ref.receiver_id), ANY_ID)
+        self._disconnect(cast(IdentityType, receiver_ref.receiver_id), ANY_ID)
 
     def _cleanup_sender(self, sender_ref: annotatable_weakref) -> None:
         """Disconnect all receivers from a sender."""
-        sender_id = t.cast(IdentityType, sender_ref.sender_id)
+        sender_id = cast(IdentityType, sender_ref.sender_id)
         assert sender_id != ANY_ID
         self._weak_senders.pop(sender_id, None)
         for receiver_id in self._by_sender.pop(sender_id, ()):
@@ -505,7 +512,7 @@ class NamedSignal(Signal):
 
     def __repr__(self) -> str:
         base = Signal.__repr__(self)
-        return f"{base[:-1]}; {self.name!r}>"
+        return f"{base[:-1]}; {self.name!r}>"  # noqa: E702
 
 
 class Namespace(dict):
