@@ -17,6 +17,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
+from collections import defaultdict, namedtuple
 import ipaddress
 import json
 import logging
@@ -39,16 +40,21 @@ JSON = Union[Dict[str, 'JSON'], List['JSON'], int, str, float, bool, Type[None]]
 DEVICE_TYPES = {
     'bond': 'bond',
     'bridge': 'bridge',
+    'dummy': 'dummy-device',
+    'erspan': 'tunnel',
     'ether': 'ethernet',
+    'gretap': 'tunnel',
     'ipgre': 'tunnel',
     'ip6gre': 'tunnel',
     'loopback': 'ethernet',
     'sit': 'tunnel',
     'tunnel': 'tunnel',
+    'tun': 'tunnel',
     'tunnel6': 'tunnel',
     'wireguard': 'tunnel',
     'wlan': 'wifi',
     'wwan': 'modem',
+    'veth': 'virtual-ethernet',
     'vlan': 'vlan',
     'vrf': 'vrf',
     'vxlan': 'tunnel',
@@ -59,7 +65,7 @@ DEVICE_TYPES = {
     'bridges': 'bridge',
     'bonds': 'bond',
     'nm-devices': 'nm-device',
-    'dummy-devices': 'dummy',
+    'dummy-devices': 'dummy-device',
     'modems': 'modem',
     'vlans': 'vlan',
     'vrfs': 'vrf',
@@ -83,6 +89,10 @@ class Interface():
         self.adminstate: str = 'UP' if 'UP' in ip.get('flags', []) else 'DOWN'
         self.operstate: str = ip.get('operstate', 'unknown').upper()
         self.macaddress: str = self.__extract_mac(ip)
+        self.bridge: str = None
+        self.bond: str = None
+        self.vrf: str = None
+        self.members: List[str] = []
 
         # Filter networkd/NetworkManager data
         nm_data = nm_data or []  # avoid 'None' value on systems without NM
@@ -212,6 +222,14 @@ class Interface():
             json['routes'] = self.routes
         if self.activation_mode:
             json['activation_mode'] = self.activation_mode
+        if self.bridge:
+            json['bridge'] = self.bridge
+        if self.bond:
+            json['bond'] = self.bond
+        if self.vrf:
+            json['vrf'] = self.vrf
+        if self.members:
+            json['interfaces'] = self.members
         return (self.name, json)
 
     @property
@@ -225,6 +243,13 @@ class Interface():
     @property
     def type(self) -> str:
         nd_type = self.nd.get('Type') if self.nd else None
+        if nd_type == 'none':
+            # If the Type is reported as 'none' by networkd, the interface still might have a Kind.
+            nd_type = self.nd.get('Kind')
+        if nd_type == 'ether':
+            # There are different kinds of 'ether' devices, such as VRFs, veth and dummies
+            if kind := self.nd.get('Kind'):
+                nd_type = kind
         if device_type := DEVICE_TYPES.get(nd_type):
             return device_type
         logging.warning('Unknown device type: {}'.format(nd_type))
@@ -327,6 +352,9 @@ class SystemConfigState():
         self.interface_list = [Interface(itf, networkd, nmcli, (dns_addresses, dns_search),
                                          (route4, route6)) for itf in iproute2]
 
+        # get bridge/bond/vrf data
+        self.correlate_members_and_uplink(self.interface_list)
+
         # show only active interfaces by default
         filtered = [itf for itf in self.interface_list if itf.operstate != 'DOWN']
         # down interfaces do not contribute anything to the online state
@@ -491,6 +519,54 @@ class SystemConfigState():
             logging.debug('Cannot query resolved DNS data: {}'.format(str(e)))
         return (addresses, search)
 
+    @classmethod
+    def query_members(cls, ifname: str) -> List[str]:
+        ''' Return a list containing the interfaces that are members of a bond/bridge/vrf '''
+        members = []
+        output: str = None
+        try:
+            output = subprocess.check_output(
+                ['ip', '-d', '-j', 'link', 'show', 'master', ifname], text=True)   # wokeignore:rule=master
+        except Exception as e:
+            logging.warning('Cannot query bridge: {}'.format(str(e)))
+            return []
+
+        output_json = json.loads(output)
+        for member in output_json:
+            members.append(member.get('ifname'))
+
+        return members
+
+    @classmethod
+    def correlate_members_and_uplink(cls, interfaces: List[Interface]) -> None:
+        '''
+        Associate interfaces with their members and parent interfaces.
+        If an interface is a member of a bond/bridge/vrf, identify which interface
+        if a member of. If an interface has members, identify what are the members.
+        '''
+        uplink_types = ['bond', 'bridge', 'vrf']
+        members_to_uplink = {}
+        uplink_to_members = defaultdict(list)
+        for interface in filter(lambda i: i.type in uplink_types, interfaces):
+            members = cls.query_members(interface.name)
+            for member in members:
+                member_tuple = namedtuple('Member', ['name', 'type'])
+                members_to_uplink[member] = member_tuple(interface.name, interface.type)
+            uplink_to_members[interface.name] = members
+
+        for interface in interfaces:
+            if uplink := members_to_uplink.get(interface.name):
+                if uplink.type == 'bridge':
+                    interface.bridge = uplink.name
+                if uplink.type == 'bond':
+                    interface.bond = uplink.name
+                if uplink.type == 'vrf':
+                    interface.vrf = uplink.name
+
+            if interface.type in uplink_types:
+                if members := uplink_to_members.get(interface.name):
+                    interface.members = members
+
     @property
     def number_of_interfaces(self) -> int:
         return len(self.interface_list)
@@ -509,6 +585,7 @@ class NetplanConfigState():
 
         np_state = netplan.State()
         np_state.import_parser_results(parser)
+        self.netdefs = np_state.netdefs
 
         self.state = StringIO()
 
